<?php

namespace Sharplogicians\Keycloak\Console\Command;

use Magento\Framework\App\State;
use Magento\Customer\Model\ResourceModel\Customer\CollectionFactory as CustomerCollectionFactory;
use Psr\Log\LoggerInterface;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Sharplogicians\Keycloak\Model\Keycloak;

class ExportCustomers extends Command
{
    /**
     * @var State
     */
    private $state;

    /**
     * @var CustomerCollectionFactory
     */
    private $customerCollectionFactory;

    /**
     * @var LoggerInterface
     */
    private $logger;

    public function __construct(
        State $state,
        CustomerCollectionFactory $customerCollectionFactory,
        LoggerInterface $logger,
        Keycloak $keycloak
    ) {
        $this->state = $state;
        $this->customerCollectionFactory = $customerCollectionFactory;
        $this->logger = $logger;
        $this->keycloak = $keycloak;     
        parent::__construct();
    }

    protected function configure()
    {
        $this->setName('keycloak:export:customers')
            ->setDescription('Export Magento customers to Keycloak');
        parent::configure();
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $output->writeln('<info>Starting customer export...</info>');

        try {
            $this->state->setAreaCode('adminhtml');
            $customers = $this->getCustomers();
            foreach ($customers as $customer) {
                $this->exportCustomerToKeycloak($customer, $output);
            }

            $output->writeln('<info>Customer export completed successfully.</info>');
        } catch (\Exception $e) {
            $output->writeln('<error>Error during export: ' . $e->getMessage() . '</error>');
            $this->logger->error('Error exporting customers to Keycloak: ' . $e->getMessage());
        }

        return Command::SUCCESS;
    }

    private function getCustomers()
    {
        $collection = $this->customerCollectionFactory->create();
        $collection->addAttributeToSelect('*'); // Fetch all customer attributes
        return $collection->getItems();
    }

    private function exportCustomerToKeycloak($customer, OutputInterface $output)
    {
        $accessToken = $this->keycloak->getAccessToken();

        if (!$accessToken) {
            $output->writeln('<error>Unable to fetch access token. Skipping export.</error>');
            return;
        }

        try {
            $response = $this->keycloak->exportCustomer($customer, $accessToken);

            if ($response->getStatusCode() === 201) {
                $output->writeln('<info>Exported customer: ' . $customer->getEmail() . '</info>');
            } else {
                $output->writeln('<error>Failed to export customer: ' . $customer->getEmail() . '</error>');
            }
        } catch (\Exception $e) {
            $output->writeln('<error>Error exporting customer: ' . $customer->getEmail() . ' - ' . $e->getMessage() . '</error>');
        }
    }
}
