<?php

namespace Sharplogicians\Keycloak\Observer;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Psr\Log\LoggerInterface;
use Sharplogicians\Keycloak\Model\Keycloak;

class ExportCustomer implements ObserverInterface
{
    /**
     * @var LoggerInterface
     */
    private $logger;

    public function __construct(
        LoggerInterface $logger,
        Keycloak $keycloak
    ) {
        $this->logger = $logger;
        $this->keycloak = $keycloak;
    }

    /**
     * Execute observer
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        /** @var CustomerInterface $customer */
        $customer = $observer->getEvent()->getCustomer();

        if (!$customer) {
            $this->logger->error('Customer registration event triggered without a customer object.');
            return;
        }

        $accessToken = $this->keycloak->getAccessToken();

        if (!$accessToken) {
            $this->logger->error('Unable to fetch access token for Keycloak.');
            return;
        }

        try {
            $this->keycloak->exportCustomer($customer, $accessToken);
        } catch (\Exception $e) {
            $this->logger->error('Error exporting customer to Keycloak: ' . $e->getMessage());
        }
    }
}
